#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <stdarg.h>
#include <unistd.h>
#include <string.h>
#include	<netdb.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/utsname.h>
#include <netinet/in.h>

/* Most of error-handling routines stolen from Stevens' books */

void            err_sys(char *str, ...);
void            err_ret(char *str, ...);
void            err_quit(char *str, ...);
char           *sys_err_str();

void
my_perror()
{
	fprintf(stderr, " %s\n", sys_err_str());
}

void
my_pinfo()
{
	fprintf(stdout, " %s\n", sys_err_str());
}

/*
 * Recoverable error.  Print a message, and return to caller. 
 *
 * err_ret(str, arg1, arg2, ...) 
 *
 * The string "str" must specify the conversion specification for any args. 
 */

/* VARARGS1 */
void
err_ret(char *str, ...)
{
	va_list         args;

	va_start(args, str);
	vfprintf(stdout, str, args);
	va_end(args);

	my_pinfo();

	fflush(stdout);
	fflush(stderr);

	return;
}

/*
 * Fatal error.  Print a message and terminate. Don't dump core and don't
 * print the system's errno value. 
 *
 * err_quit(str, arg1, arg2, ...) 
 *
 * The string "str" must specify the conversion specification for any args. 
 */

/* VARARGS1 */
void
err_quit(char *str, ...)
{
	va_list         args;

	va_start(args, str);
	vfprintf(stderr, str, args);
	fputc('\n', stderr);
	va_end(args);

	exit(1);
}

/*
 * Fatal error related to a system call.  Print a message and terminate.
 * Don't dump core, but do print the system's errno value and its associated
 * message. 
 *
 * err_sys(str, arg1, arg2, ...) 
 *
 * The string "str" must specify the conversion specification for any args. 
 */

/* VARARGS1 */
void
err_sys(char *str, ...)
{
	va_list         args;

	va_start(args, str);
	vfprintf(stderr, str, args);
	va_end(args);

	my_perror();

	exit(1);
}

/*
 * Return a string containing some additional operating-system dependent
 * information. Note that different versions of UNIX assign different
 * meanings to the same value of "errno" (compare errno's starting with 35
 * between System V and BSD, for example).  This means that if an error
 * condition is being sent to another UNIX system, we must interpret the
 * errno value on the system that generated the error, and not just send the
 * decimal value of errno to the other system. 
 */

char           *
sys_err_str()
{
	static char     msgstr[200];

	if (errno != 0) {
		sprintf(msgstr, "(%s)", strerror(errno));
	} else {
		msgstr[0] = '\0';
	}
	return (msgstr);
}

void
test_option_int(const char *label, int optname, int optval)
{
	int             mysocket;
	int             result;
	mysocket = socket(PF_INET, SOCK_DGRAM, AF_UNSPEC);
	if (mysocket == -1) {
		err_sys("Cannot create socket: ");
	}
	result = setsockopt(mysocket, IPPROTO_IP, optname, &optval, sizeof(int));
	if (result != 0) {
		err_ret("[PROBLEM] Cannot set the %s to %i: ", label, optval);
	} else {
		fprintf(stdout, "%s successfully set to %i\n", label, optval);
	}
	close(mysocket);
}

int
main(int argc, char **argv)
{
	int             myttl = 10;
#ifdef IPTOS_THROUGHPUT
	int             mytos = IPTOS_THROUGHPUT;
#else
	int             mytos = 0x08;	/* Maximize throughput */
#endif
#ifdef IPTOS_PREC_CRITIC_ECP
	int             myprecedence = IPTOS_PREC_CRITIC_ECP;
#else
	int             myprecedence = 0xa0;	/* Raise the precedence to "critic" */
#endif
	int             myecn = 0x02;	/* ECN-capable transport */
	uid_t           myuid;
	int             result;
	struct utsname  system_info;
	myuid = getuid();
	fprintf(stdout, "Testing of the abilities to set bits in the IP header\n");
	fprintf(stdout, "My UID is %i\n", myuid);
	result = uname(&system_info);
	if (result != 0) {
		err_sys("Cannot call uname: ");
	}
	fprintf(stdout, "The system is %s %s, the machine is %s\n",
		system_info.sysname, system_info.release, system_info.machine);
	test_option_int("TTL", IP_TTL, myttl);
	test_option_int("TOS", IP_TOS, mytos);
	test_option_int("ECN", IP_TOS, myecn);
	test_option_int("precedence", IP_TOS, myprecedence);
#ifdef IP_MTU_DISCOVER
	/* To set the DF bit, we hack a little, going through the PMTU options. Not
	 * portable. */
	test_option_int("DF set through PMTU", IP_MTU_DISCOVER, IP_PMTUDISC_DO);
	test_option_int("DF clear through PMTU", IP_MTU_DISCOVER, IP_PMTUDISC_DONT);
#else
	fprintf(stdout, "No known way to set the DF bit on this system\n");
#endif
	fprintf(stdout,
		"\nWarning: we only tested that setsockopt was happy,\n\tnot that the actual packet headers were changed\n");
	exit(0);
}
