#include "common.h"

int             mysocket;

char           *reception_port_name;
int             reception_port;

int
ttl_type(struct cmsghdr control)
{
	/* On FreeBSD, we receive type IP_RECVTTL and not IP_TTL. Probably a bug. */
#ifdef IP_RECVTTL
	return (control.cmsg_type == IP_TTL || control.cmsg_type == IP_RECVTTL);
#else
	return (control.cmsg_type == IP_TTL);	/* Probably useless, if we do not
						 * have IP_RECVTTL for setsockopt... 
						 */
#endif
}

void
test_option()
{
	int             result;
	struct msghdr  *message = malloc(sizeof(struct msghdr));
	struct cmsghdr *c_msg;
	char           *buffer = malloc(MAX_DATA);
	int             length = MAX_DATA;
	char           *control_buffer = malloc(MAX_DATA);
	int             control_length = MAX_DATA;

	memset(message, 0, sizeof(*message));
	message->msg_iov = malloc(sizeof(struct iovec));
	message->msg_iov->iov_base = buffer;
	message->msg_iov->iov_len = length;
	message->msg_iovlen = 1;
	/* Message control does not exist on every system. For instance, HP Tru64
	 * does not have it */
#ifndef NO_MESSAGE_CONTROL
	message->msg_control = control_buffer;
	message->msg_controllen = control_length;
#endif
	result = recvmsg(mysocket, message, 0);
	if (result <= 0) {
		err_ret("[PROBLEM] Cannot receive data (%s)", sys_err_str());
	} else {
		fprintf(stdout, "Received %i bytes\n", result);
#ifndef NO_MESSAGE_CONTROL
		/* See cmsg(3) */
		for (c_msg = CMSG_FIRSTHDR(message); c_msg;
		     c_msg = (CMSG_NXTHDR(message, c_msg))) {
			if ((c_msg->cmsg_level == IPPROTO_IP && ttl_type(*c_msg))
			    || (c_msg->cmsg_level == IPPROTO_IPV6
				&& c_msg->cmsg_type == IPV6_HOPLIMIT)) {
				fprintf(stdout, "\tThe TTL is %i\n",
					*(int *) CMSG_DATA(c_msg));
			} else if (c_msg->cmsg_level == IPPROTO_IP
				   && c_msg->cmsg_type == IP_TOS) {
				fprintf(stdout, "\tThe TOS/DSCP is %i\n",
					*(int *) CMSG_DATA(c_msg));
			} else if (c_msg->cmsg_level == IPPROTO_IP
				   && c_msg->cmsg_type == IP_OPTIONS) {
				fprintf(stdout, "\tThe IP options are %x\n",
					*(int *) CMSG_DATA(c_msg));
			} else {
				fprintf(stderr,
					"\tWarning, unexpected data of level %i and type %i\n",
					c_msg->cmsg_level, c_msg->cmsg_type);
			}
		}
#else
		fprintf(stdout,
			"No message control on that platform, so no way to find IP options\n");
#endif
	}
}

int
main(int argc, char **argv)
{
	uid_t           myuid;
	int             result;
	const int       One = 1;
	struct utsname  system_info;
	/* TODO: IPv6, too */
	struct sockaddr_in *my_addr = malloc(sizeof(struct sockaddr_in));

	if (argc != 2)
		err_quit("Usage: %s reception_port", argv[0]);
	reception_port_name = malloc(NI_MAXSERV);
	strcpy(reception_port_name, argv[1]);
	reception_port = atoi(reception_port_name);

	myuid = getuid();
	fprintf(stdout, "Testing of the abilities to read bits in the IP header\n");
	fprintf(stdout, "My UID is %i\n", myuid);
	result = uname(&system_info);
	if (result != 0) {
		err_sys("Cannot call uname: ");
	}
	fprintf(stdout, "The system is %s %s, the machine is %s\n",
		system_info.sysname, system_info.release, system_info.machine);

	mysocket = socket(PF_INET, SOCK_DGRAM, AF_UNSPEC);
	if (mysocket == -1) {
		err_sys("Cannot create socket: ");
	}
	memset(my_addr, 0, sizeof(*my_addr));
	my_addr->sin_family = AF_INET;
	my_addr->sin_port = htons(reception_port);
	result = bind(mysocket, (struct sockaddr *) my_addr, sizeof(*my_addr));
	if (result == -1) {
		err_sys("Cannot bind socket: ");
	}
#ifdef IP_RECVTTL
	result = setsockopt(mysocket, IPPROTO_IP, IP_RECVTTL, &One, sizeof(One));
	if (result != 0) {
		fprintf(stderr, "[PROBLEM] Cannot set the socket to TTL reception");
	}
#else
	fprintf(stdout,
		"No way to ask the TTL of incoming packets on that platform.\n");
#endif
#ifdef IP_RECVTOS
	result = setsockopt(mysocket, IPPROTO_IP, IP_RECVTOS, &One, sizeof(One));
	if (result != 0) {
		fprintf(stderr,
			"[PROBLEM] Cannot set the socket to TOS/DSCP reception");
	}
#else
	fprintf(stdout,
		"No way to ask the TOS/DSCP of incoming packets on that platform.\n");
#endif
	result = setsockopt(mysocket, IPPROTO_IP, IP_RECVOPTS, &One, sizeof(One));
	if (result != 0) {
		fprintf(stderr,
			"[PROBLEM] Cannot set the socket to IP options reception");
	}

	test_option();
	test_option();
	test_option();
	test_option();
	test_option();
	test_option();
	close(mysocket);
	exit(0);
}
