#include "common.h"

char           *dest_host;
int             dest_port;
char           *dest_port_name;
struct addrinfo *destination_info;

void
test_option_int(const char *label, int optname, int optval)
{
	int             mysocket;
	int             result;

	mysocket = socket(PF_INET, SOCK_DGRAM, AF_UNSPEC);
	if (mysocket == -1) {
		err_sys("Cannot create socket: ");
	}
	result =
	    connect(mysocket, destination_info->ai_addr,
		    destination_info->ai_addrlen);
	if (result != 0) {
		err_ret("[PROBLEM] Cannot connect to %s:%i: ", dest_host, dest_port);
	}
	result = setsockopt(mysocket, IPPROTO_IP, optname, &optval, sizeof(int));
	if (result != 0) {
		fprintf(stderr, "[PROBLEM] Cannot set the %s to %i: \n", label,
			optval);
		/* But send anyway */
	} else {
		fprintf(stdout, "%s successfully set to %i\n", label, optval);
	}
	result = send(mysocket, DATA, sizeof(DATA), 0);
	if (result != sizeof(DATA)) {
		err_ret("[PROBLEM] Cannot send data to %s:%i: %s", dest_host,
			dest_port, sys_err_str());
	}
	close(mysocket);
}

int
main(int argc, char **argv)
{
	uid_t           myuid;
	int             result;
	struct utsname  system_info;
	struct addrinfo hints;

	if (argc != 3)
		err_quit("Usage: %s destination_host destination_port ", argv[0]
		    );
	dest_host = malloc(MAX_HOSTNAME);
	dest_port_name = malloc(NI_MAXSERV);
	strcpy(dest_host, argv[1]);
	strcpy(dest_port_name, argv[2]);
	dest_port = atoi(dest_port_name);
	memset(&hints, 0, sizeof(hints));
	/* We force IPv4 (AF_INET) because of the receiver which is
	   v4-only. It is not easy, with the socket interface, to make a
	   multi-protocol server. */
	hints.ai_family = AF_INET;
	hints.ai_socktype = SOCK_DGRAM;
	result = getaddrinfo(dest_host, dest_port_name, &hints, &destination_info);
	if (result != 0) {
		err_quit("getaddrinfo error for host \"%s\" (%s)",
			 dest_host, gai_strerror(result));
	}

	myuid = getuid();
	fprintf(stdout, "Testing of the abilities to set bits in the IP header\n");
	fprintf(stdout, "My UID is %i\n", myuid);
	result = uname(&system_info);
	if (result != 0) {
		err_sys("Cannot call uname: ");
	}
	fprintf(stdout, "The system is %s %s, the machine is %s\n",
		system_info.sysname, system_info.release, system_info.machine);

	test_option_int("TTL", IP_TTL, myttl);
	test_option_int("TOS", IP_TOS, mytos);
	test_option_int("ECN", IP_TOS, myecn);
	test_option_int("precedence", IP_TOS, myprecedence);
#ifdef IP_MTU_DISCOVER
	/* To set the DF bit, we hack a little, going through the PMTU options. Not
	 * portable. */
	test_option_int("DF set through PMTU", IP_MTU_DISCOVER, IP_PMTUDISC_DO);
	test_option_int("DF clear through PMTU", IP_MTU_DISCOVER, IP_PMTUDISC_DONT);
#else
	fprintf(stdout, "No known way to set the DF bit on this system\n");
#endif
	exit(0);
}
